/* documentpageview.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gtk;

public class I4uc.GtkFrontend.DocumentPageView : Page, I4uc.Core.DocumentPageView
{
	private App _app;
	private MessageBar _error_message_bar = new MessageBar ();
	private MessageBar _reload_message_bar = new MessageBar ();
	private MessageBar _save_message_bar = new MessageBar ();
	
	//Search and Replace bar
	private SearchBarView _search_bar_view;
	
	//Content
	private SourceView _source_view;
	private SourceBuffer _source_buffer = new SourceBuffer (null);
	
	public string font_name
	{
		set { _source_view.modify_font (Pango.FontDescription.from_string (value)); }
		get { return ""; }
	}
	
	public uint tab_width { set; get; }
	public bool auto_indent { set; get; }
	public bool highlight_current_line { set; get; }
	public bool insert_spaces_instead_of_tabs { set; get; }
	public bool show_line_numbers { set; get; }
	public bool highlight_matching_brackets { set; get; }
	
	public bool wrap_mode
	{
		set { _source_view.wrap_mode = value ? WrapMode.WORD_CHAR : WrapMode.NONE; }
		get { return _source_view.wrap_mode == WrapMode.WORD_CHAR; }
	}
	
	public bool can_undo { get { return _source_buffer.can_undo; } }
	public bool can_redo { get { return _source_buffer.can_redo; } }
	public bool has_selection { get { return _source_buffer.has_selection; } }
	
	public string content
	{
		set
		{
			_source_buffer.begin_not_undoable_action ();
			
			var content_size = value.size ();
			if (value.has_suffix ("\n"))
				content_size--;
			
			_source_buffer.set_text (value, (int)content_size);
			_source_buffer.set_modified (false);
			_source_buffer.end_not_undoable_action ();
			
			TextIter start;
			_source_buffer.get_start_iter (out start);
			_source_buffer.place_cursor (start);
		}
		owned get
		{
			TextIter start, end;
			_source_buffer.get_bounds (out start, out end);
			return _source_buffer.get_text (start, end, false);
		}
	}
	
	public string language_id
	{
		set
		{
			if (value == "none")
				_source_buffer.language = null;
			else
				_source_buffer.language = SourceLanguageManager.get_default ().get_language (value);
		}
		get
		{
			if (_source_buffer.language == null)
				return "none";
			return _source_buffer.language.id;
		}
	}
	
	public I4uc.Core.MessageBar error_message_bar { get { return _error_message_bar; } }
	public I4uc.Core.MessageBar reload_message_bar { get { return _reload_message_bar; } }
	public I4uc.Core.MessageBar save_message_bar { get { return _save_message_bar; } }
	
	public I4uc.Core.SearchBarView search_bar_view { get { return _search_bar_view; } }
	
	public bool search_bar_view_visible { set; get; }
	public bool content_visible { set; get; }
	
	public bool content_modified
	{
		set { _source_buffer.set_modified (value); }
		get { return _source_buffer.get_modified (); }
	}
	
	public DocumentPageView (App app)
	{
		base.with_close_button ();
		
		_app = app;
		
		create_error_bar ();
		create_reload_bar ();
		create_save_bar ();
		
		_source_view = new SourceView.with_buffer (_source_buffer);
		_search_bar_view = new SearchBarView (_source_view);
		
		var scrolled_window = new ScrolledWindow (null, null);
		scrolled_window.add (_source_view);
		scrolled_window.set_policy (PolicyType.AUTOMATIC, PolicyType.AUTOMATIC);
		
		pack_start (_error_message_bar, false, false, 2);
		pack_start (_reload_message_bar, false, false, 2);
		pack_start (_save_message_bar, false, false, 2);
		pack_start (_search_bar_view, false, false, 2);
		pack_start (scrolled_window, true, true, 0);
		
		//Bind properties
		bind_property ("tab-width", _source_view, "tab-width", BindingFlags.BIDIRECTIONAL);
		bind_property ("auto-indent", _source_view, "auto-indent", BindingFlags.BIDIRECTIONAL);
		bind_property ("highlight-current-line", _source_view, "highlight-current-line", BindingFlags.BIDIRECTIONAL);
		bind_property ("insert-spaces-instead-of-tabs", _source_view, "insert-spaces-instead-of-tabs", BindingFlags.BIDIRECTIONAL);
		bind_property ("show-line-numbers", _source_view, "show-line-numbers", BindingFlags.BIDIRECTIONAL);
		bind_property ("highlight-matching-brackets", _source_buffer, "highlight-matching-brackets", BindingFlags.BIDIRECTIONAL);
		bind_property ("search-bar-view-visible", _search_bar_view, "visible", BindingFlags.BIDIRECTIONAL);
		bind_property ("content-visible", _source_view, "visible", BindingFlags.BIDIRECTIONAL);
		
		//Connect signals
		_error_message_bar.response.connect (() => this.error_message_bar.ok_clicked ());
		_reload_message_bar.response.connect (on_reload_bar_response);
		_save_message_bar.response.connect (on_save_bar_response);
		_source_buffer.changed.connect (() => this.content_changed ());
		_source_buffer.modified_changed.connect (() => this.modified_changed ());
		_source_buffer.notify["has-selection"].connect (() => this.selection_changed ());
		_source_view.focus_in_event.connect (() => {
			this.focused_in ();
			return false;
		});
	}
	
	public void undo ()
	{
		if (_source_buffer.can_undo)
			_source_buffer.undo ();
	}
	
	public void redo ()
	{
		if (_source_buffer.can_redo)
			_source_buffer.redo ();
	}
	
	public void cut ()
	{
		var clipboard = _source_view.get_clipboard (Gdk.SELECTION_CLIPBOARD);
		_source_buffer.cut_clipboard (clipboard, true);
	}
	
	public void copy ()
	{
		var clipboard = _source_view.get_clipboard (Gdk.SELECTION_CLIPBOARD);
		_source_buffer.copy_clipboard (clipboard);
	}
	
	public void paste ()
	{
		var clipboard = _source_view.get_clipboard (Gdk.SELECTION_CLIPBOARD);
		_source_buffer.paste_clipboard (clipboard, null, true);
	}
	
	public void guess_language_id (string content_type)
	{
		var lang_manager = SourceLanguageManager.get_default ();
		_source_buffer.language = lang_manager.guess_language (null, content_type);
	}
	
	public void show_print_dialog () throws GLib.Error
	{
		var print_compositor = new SourcePrintCompositor (_source_buffer);
		print_compositor.tab_width = _app.settings.tab_width;
		print_compositor.print_line_numbers = _app.settings.show_line_numbers ? 1 : 0;
		print_compositor.body_font_name = _app.settings.font;
		print_compositor.set_header_format (true, null, this.title, null);
		print_compositor.set_footer_format (true, null, _("Page %N of %Q"), null);
		print_compositor.print_header = true;
		print_compositor.print_footer = true;
		print_compositor.wrap_mode = WrapMode.WORD_CHAR;
		
		var operation = new PrintOperation ();
		operation.print_settings = _app.print_settings;
		operation.default_page_setup = _app.page_setup;
		
		operation.paginate.connect ((operation, context) => {
			if (print_compositor.paginate (context))
			{
				operation.n_pages = print_compositor.n_pages;
				return true;
			}
			return false;
		});
		operation.draw_page.connect ((operation, context, page) => {
			print_compositor.draw_page (context, page);
		});
		operation.end_print.connect (() => {
			print_compositor = null;
		});
		
		operation.run (PrintOperationAction.PRINT_DIALOG, null);
	}
	
	public I4uc.Core.DialogResponse show_close_message ()
	{
		var message = new MessageDialog (null, DialogFlags.MODAL, MessageType.WARNING, ButtonsType.NONE, _("Do you want to save the file <<%s>>?"), this.tab_title);
		message.add_button (STOCK_CANCEL, ResponseType.CANCEL);
		message.add_button (STOCK_NO, ResponseType.NO);
		message.add_button (STOCK_YES, ResponseType.YES);
		message.set_default_response (ResponseType.YES);
		var response = message.run ();
		message.destroy ();
		return DialogResponses.get_dialog_response ((ResponseType)response);
	}
	
	public new void grab_focus ()
	{
		_source_view.grab_focus ();
	}
	
	public void go_to_line (int line)
	{
		TextIter iter;
		
		if (line >= _source_buffer.get_line_count ())
			_source_buffer.get_end_iter (out iter);
		else
			_source_buffer.get_iter_at_line (out iter, line-1);
		
		_source_buffer.place_cursor (iter);
		_source_view.scroll_to_iter (iter, 0.25, false, 0, 0);
	}
	
	private void create_error_bar ()
	{
		_error_message_bar.set_message_type (MessageType.ERROR);
		_error_message_bar.add_button (STOCK_OK, ResponseType.OK);
	}
	
	private void create_reload_bar ()
	{
		_reload_message_bar.set_message_type (MessageType.WARNING);
		_reload_message_bar.add_button (STOCK_REFRESH, ResponseType.OK);
		_reload_message_bar.add_button (STOCK_CANCEL, ResponseType.CANCEL);
	}
	
	private void on_reload_bar_response (int response)
	{
		if (response == ResponseType.OK)
			this.reload_message_bar.ok_clicked ();
		
		if (response == ResponseType.CANCEL)
			this.reload_message_bar.cancel_clicked ();
	}

	private void create_save_bar ()
	{
		_save_message_bar.set_message_type (MessageType.WARNING);
		_save_message_bar.add_button (STOCK_SAVE, ResponseType.OK);
		_save_message_bar.add_button (STOCK_CANCEL, ResponseType.CANCEL);
	}
	
	private void on_save_bar_response (int response)
	{
		if (response == ResponseType.OK)
			this.save_message_bar.ok_clicked ();
		
		if (response == ResponseType.CANCEL)
			this.save_message_bar.cancel_clicked ();
	}
}
