/* documentsview.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gtk;
using Gee;

public class I4uc.GtkFrontend.DocumentsView : GLib.Object, I4uc.Core.DocumentsView
{
	private UIManager _ui_manager;
	private PagesPanel _pages_panel;
	private PagesPanel _side_panel;
	private App _app;
	private SyntaxHighlightMenu _syntax_highlight_menu;
	private DocumentPreferencesPageView _preferences_page_view = new DocumentPreferencesPageView ();
	private Gtk.ActionGroup _action_group;
	
	public bool save_sensitive { set; get; }
	public bool save_as_sensitive { set; get; }
	public bool print_sensitive { set; get; }
	public bool undo_sensitive { set; get; }
	public bool redo_sensitive { set; get; }
	public bool cut_sensitive { set; get; }
	public bool copy_sensitive { set; get; }
	public bool paste_sensitive { set; get; }
	public bool search_sensitive { set; get; }
	public bool save_all_sensitive { set; get; }
	public bool close_all_sensitive { set; get; }
	public bool previous_sensitive { set; get; }
	public bool next_sensitive { set; get; }
	
	public I4uc.Core.PagesPanel pages_panel { get { return _pages_panel; } }
	public I4uc.Core.SyntaxHighlightMenu syntax_highlight_menu { get { return _syntax_highlight_menu; } }
	public I4uc.Core.DocumentPreferencesPageView preferences_page_view { get { return _preferences_page_view; } }
	
	public bool syntax_highlight_menu_sensitive { set; get; }
	
	public DocumentsView (UIManager ui_manager, PagesPanel pages_panel, PagesPanel side_panel, App app)
	{
		_ui_manager = ui_manager;
		_pages_panel = pages_panel;
		_side_panel = side_panel;
		_app = app;
		
		_action_group = new Gtk.ActionGroup ("I4ucDocumentsActions");
		_action_group.set_translation_domain (Config.GETTEXT_PACKAGE);
		_action_group.add_actions (_action_entries, this);
		
		//Add recent button to the file menu
		var recent_filter = new RecentFilter ();
		recent_filter.add_application (Environment.get_application_name ());
		
		var recent_action = new RecentAction ("OpenRecentDocumentAction", _("Open _Recent"), _("Open recently used files"), "");
		recent_action.show_icons = false;
		recent_action.filter = recent_filter;
		recent_action.sort_type = RecentSortType.MRU;
		_action_group.add_action (recent_action);
		
		//Add recent button to the toolbar
		var toolbar_recent_menu = new RecentChooserMenu.for_manager (RecentManager.get_default ());
		toolbar_recent_menu.show_icons = false;
		toolbar_recent_menu.filter = recent_filter;
		toolbar_recent_menu.sort_type = RecentSortType.MRU;
				
		var open_button = new MenuToolButton.from_stock (STOCK_OPEN);
		open_button.menu = toolbar_recent_menu;
		open_button.tooltip_text = _("Open a file");
		open_button.related_action = _action_group.get_action ("OpenDocumentAction");
		
		_ui_manager.insert_action_group (_action_group, -1);
		
		try
		{
			_ui_manager.add_ui_from_string (_UI, -1);
		}
		catch (GLib.Error e)
		{
			warning (e.message);
		}
		
		var toolbar = (Toolbar)_ui_manager.get_widget ("/MainToolbar");
		toolbar.insert (open_button, 3);
		toolbar.set_style (ToolbarStyle.ICONS);
		
		_syntax_highlight_menu = new SyntaxHighlightMenu (_ui_manager, _action_group);
		
		_side_panel.add_page (_preferences_page_view);
		
		//Bind properties
		bind_property ("save-sensitive", _action_group.get_action ("SaveDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("save-as-sensitive", _action_group.get_action ("SaveAsDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("print-sensitive", _action_group.get_action ("PrintDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("undo-sensitive", _action_group.get_action ("UndoDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("redo-sensitive", _action_group.get_action ("RedoDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("cut-sensitive", _action_group.get_action ("CutDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("copy-sensitive", _action_group.get_action ("CopyDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("paste-sensitive", _action_group.get_action ("PasteDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("search-sensitive", _action_group.get_action ("SearchDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("save-all-sensitive", _action_group.get_action ("SaveAllDocumentsAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("close-all-sensitive", _action_group.get_action ("CloseAllDocumentsAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("previous-sensitive", _action_group.get_action ("PreviousDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("next-sensitive", _action_group.get_action ("NextDocumentAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		bind_property ("syntax-highlight-menu-sensitive", _action_group.get_action ("SyntaxHighlightMenuAction"), "sensitive", BindingFlags.BIDIRECTIONAL);
		
		//Signals
		recent_action.item_activated.connect ((chooser) => this.open_recent_clicked (chooser.get_current_uri ()));
		toolbar_recent_menu.item_activated.connect ((chooser) => this.open_recent_clicked (chooser.get_current_uri ()));
	}
	
	public I4uc.Core.DocumentPageView create_page_view ()
	{
		var page_view = new DocumentPageView (_app);
		page_view.show_all ();
		return page_view;
	}
	
	public void show_page_setup_dialog ()
	{
		var new_page_setup = print_run_page_setup_dialog (null, _app.page_setup, _app.print_settings);
		_app.page_setup = new_page_setup;
	}
	
	public I4uc.Core.DialogResponse show_close_confirmation_dialog (Gee.Map<string, string> unsaved_documents, out Gee.List<string> documents_to_save)
	{
		var dialog = new CloseConfirmationDialog (unsaved_documents);
		var response = dialog.run ();
		documents_to_save = dialog.list_selected_documents ();
		dialog.destroy ();
		return DialogResponses.get_dialog_response ((ResponseType)response);
	}
	
	private void on_new ()
	{
		this.new_clicked ();
	}
	
	private void on_open ()
	{
		this.open_clicked ();
	}
	
	private void on_save ()
	{
		this.save_clicked ();
	}
	
	private void on_save_as ()
	{
		this.save_as_clicked ();
	}
	
	private void on_print ()
	{
		this.print_clicked ();
	}
	
	private void on_page_setup ()
	{
		this.page_setup_clicked ();
	}
	
	private void on_undo ()
	{
		this.undo_clicked ();
	}
	
	private void on_redo ()
	{
		this.redo_clicked ();
	}
	
	private void on_cut ()
	{
		this.cut_clicked ();
	}
	
	private void on_copy ()
	{
		this.copy_clicked ();
	}
	
	private void on_paste ()
	{
		this.paste_clicked ();
	}
	
	private void on_search ()
	{
		this.search_clicked ();
	}
	
	private void on_save_all ()
	{
		this.save_all_clicked ();
	}
	
	private void on_close_all ()
	{
		this.close_all_clicked ();
	}
	
	private void on_previous ()
	{
		this.previous_clicked ();
	}
	
	private void on_next ()
	{
		this.next_clicked ();
	}
	
	private const ActionEntry[] _action_entries =
	{
		{ "DocumentsMenuAction", null, N_("_Documents") },
		{ "SyntaxHighlightMenuAction", null, N_("Syntax Hightlight") },
		{ "NewDocumentAction", STOCK_NEW, null, null, N_("Create a new document"), on_new },
		{ "OpenDocumentAction", STOCK_OPEN, null, null, N_("Open a file"), on_open },
		{ "SaveDocumentAction", STOCK_SAVE, null, null, N_("Save current document"), on_save },
		{ "SaveAsDocumentAction", STOCK_SAVE_AS, null, null, N_("Save current document with a diferent name"), on_save_as },
		{ "PrintDocumentAction", STOCK_PRINT, null, null, N_("Print current document"), on_print },
		{ "PageSetupDocumentAction", STOCK_PAGE_SETUP, null, null, N_("Setup page for print"), on_page_setup },
		{ "UndoDocumentAction", STOCK_UNDO, null, "<Control>z", N_("Undo last operation"), on_undo },
		{ "RedoDocumentAction", STOCK_REDO, null, "<Control><Shift>z", N_("Redo last undone operation"), on_redo },
		{ "CutDocumentAction", STOCK_CUT, null, null, N_("Cut seletion"), on_cut },
		{ "CopyDocumentAction", STOCK_COPY, null, null, N_("Copy selection"), on_copy },
		{ "PasteDocumentAction", STOCK_PASTE, null, null, N_("Paste from clipboard"), on_paste },
		{ "SearchDocumentAction", STOCK_FIND, null, null, N_("Search for a text in current document"), on_search },
		{ "SaveAllDocumentsAction", STOCK_SAVE, N_("Save all"), "<Control><Shift>l", N_("Save all documents"), on_save_all },
		{ "CloseAllDocumentsAction", STOCK_CLOSE, N_("Close all"), "<Control><Shift>w", N_("Close all documents"), on_close_all },
		{ "PreviousDocumentAction", STOCK_GO_BACK, N_("Previous document"), "<Control><Alt>Page_Up", N_("Activate previous document"), on_previous },
		{ "NextDocumentAction", STOCK_GO_FORWARD, N_("Next document"), "<Control><Alt>Page_Down", N_("Activate next document"), on_next }
	};
	
	private const string _UI = """
<ui>
	<menubar name="MainMenu">
		<menu name="FileMenu" action="FileMenuAction">
			<placeholder name="FileMenuOps">
				<separator/>
				<menuitem action="NewDocumentAction"/>
				<menuitem action="OpenDocumentAction"/>
				<menuitem action="OpenRecentDocumentAction"/>
				<separator/>
				<menuitem action="SaveDocumentAction"/>
				<menuitem action="SaveAsDocumentAction"/>
				<separator/>
				<menuitem action="PrintDocumentAction"/>
				<menuitem action="PageSetupDocumentAction"/>
				<separator/>
			</placeholder>
		</menu>
		<menu name="EditMenu" action="EditMenuAction">
			<placeholder name="EditMenuOps">
				<separator/>
				<menuitem action="UndoDocumentAction"/>
				<menuitem action="RedoDocumentAction"/>
				<separator/>
				<menuitem action="CutDocumentAction"/>
				<menuitem action="CopyDocumentAction"/>
				<menuitem action="PasteDocumentAction"/>
				<separator/>
				<menuitem action="SearchDocumentAction"/>
				<separator/>
			</placeholder>
		</menu>
		<menu name="ViewMenu" action="ViewMenuAction">
			<placeholder name="ViewMenuOps">
				<separator/>
				<menu name="SyntaxHighlightMenu" action="SyntaxHighlightMenuAction"/>
				<separator/>
			</placeholder>
		</menu>
		<placeholder name="MenuBarOps">
			<menu name="DocumentsMenu" action="DocumentsMenuAction">
				<menuitem action="SaveAllDocumentsAction"/>
				<menuitem action="CloseAllDocumentsAction"/>
				<separator/>
				<menuitem action="PreviousDocumentAction"/>
				<menuitem action="NextDocumentAction"/>
			</menu>
		</placeholder>
	</menubar>
	<toolbar name="MainToolbar">
		<placeholder name="ToolbarStartOps">
			<separator/>
			<toolitem action="NewDocumentAction"/>
			<toolitem action="SaveDocumentAction"/>
			<separator/>
			<toolitem action="PrintDocumentAction"/>
			<separator/>
			<toolitem action="UndoDocumentAction"/>
			<toolitem action="RedoDocumentAction"/>
			<separator/>
			<toolitem action="CutDocumentAction"/>
			<toolitem action="CopyDocumentAction"/>
			<toolitem action="PasteDocumentAction"/>
			<separator/>
			<toolitem action="SearchDocumentAction"/>
			<separator/>
		</placeholder>
	</toolbar>
</ui>""";
}
