/* optionview.vala
 *
 * Copyright (C) 2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gtk;
using Gee;

public abstract class I4uc.GtkFrontend.OptionView : Gtk.VBox, I4uc.Core.OptionView
{
	protected string _name;
	protected bool _optional;
	protected Label _label;
	protected CheckButton _active_check;
	
	public new string name { get { return _name; } }
	public string tooltip { set; get; }
	
	public abstract string value { set; owned get; }
	
	public bool active { set; get; }
	public bool value_sensitive { set; get; }
	
	public OptionView (string name, string label, bool optional)
	{
		_name = name;
		_optional = optional;
		_label = new Label (label + ":");
		_label.xalign = 0.0f;
		
		// Bind properties
		bind_property ("tooltip", _label, "tooltip-markup", BindingFlags.BIDIRECTIONAL);
		if (_optional)
		{
			_active_check = new CheckButton ();
			bind_property ("active", _active_check, "active", BindingFlags.BIDIRECTIONAL);
		}
	}
	
	protected void pack_value (Widget widget)
	{
		// Bind value_sensitive
		if (_optional)
			bind_property ("value-sensitive", widget, "sensitive", BindingFlags.BIDIRECTIONAL);
		
		// Pack label and value widgets
		pack_start (_label, false, false, 0);
		var hbox = new HBox (false, 0);
		if (_optional)
			hbox.pack_start (_active_check, false, false, 0);
		hbox.pack_start (widget, false, false, 0);
		pack_start (hbox);
	}
}

public class I4uc.GtkFrontend.NoneOptionView : OptionView
{
	public override string value
	{
		set { }
		owned get { return _active_check.active.to_string (); }
	}
	
	public NoneOptionView (string name, string label)
	{
		base (name, label, true);
		
		var hbox = new HBox (false, 0);
		hbox.pack_start (_label, false, false, 0);
		hbox.pack_start (_active_check, false, false, 0);
		pack_start (hbox);
	}
}

public class I4uc.GtkFrontend.EntryOptionView : OptionView
{
	private Entry _value_entry = new Entry ();
	
	public override string value { set; owned get; }
	
	public EntryOptionView (string name, string label, bool optional)
	{
		base (name, label, optional);
		
		// Bind properties
		bind_property ("value", _value_entry, "text", BindingFlags.BIDIRECTIONAL);
		
		pack_value (_value_entry);
	}
}

public class I4uc.GtkFrontend.ListOptionView : OptionView, I4uc.Core.ListOptionView
{
	private ComboBox _value_combo = new ComboBox.text ();
	private ArrayList<string> _items = new ArrayList<string> ();
	
	public override string value
	{
		set { _value_combo.active = value in _items ? _items.index_of (value) : -1; }
		owned get
		{
			if (_value_combo.active == -1)
				return "";
			return _value_combo.get_active_text ();
		}
	}
	
	public ListOptionView (string name, string label, bool optional)
	{
		base (name, label, optional);
		
		_value_combo.changed.connect (() => notify_property ("value"));
		
		pack_value (_value_combo);
	}
	
	public void add_items (Gee.List<string> items)
	{
		foreach (var item in items)
		{
			_value_combo.append_text (item);
			_items.add (item);
		}
	}
	
	public void clear_items ()
	{
		(_value_combo.model as ListStore).clear ();
		_items.clear ();
	}
}

public class I4uc.GtkFrontend.FilePathOptionView : OptionView
{
	private FileChooserButton _value_chooser;
	
	public override string value
	{
		set { _value_chooser.set_filename (value); }
		owned get { return _value_chooser.get_filename (); }
	}
	
	public FilePathOptionView (string name, string label, bool optional)
	{
		base (name, label, optional);
		_value_chooser = new FileChooserButton (label, FileChooserAction.OPEN);
		
		_value_chooser.selection_changed.connect (() => notify_property ("value"));
		
		pack_value (_value_chooser);
	}
}

public class I4uc.GtkFrontend.FolderPathOptionView : OptionView
{
	private FileChooserButton _value_chooser;
	
	public override string value
	{
		set { _value_chooser.set_current_folder (value); }
		owned get { return _value_chooser.get_current_folder (); }
	}
	
	public FolderPathOptionView (string name, string label, bool optional)
	{
		base (name, label, optional);
		_value_chooser = new FileChooserButton (label, FileChooserAction.SELECT_FOLDER);
		
		_value_chooser.selection_changed.connect (() => notify_property ("value"));
		
		pack_value (_value_chooser);
	}
}

public class I4uc.GtkFrontend.RangeOptionView : OptionView, I4uc.Core.RangeOptionView
{
	private SpinButton _value_spin = new SpinButton.with_range (0.0, 10.0, 1.0);
	
	public override string value
	{
		set { _value_spin.value = value.to_double (); }
		owned get { return _value_spin.value.to_string (); }
	}
	
	public RangeOptionView (string name, string label, bool optional)
	{
		base (name, label, optional);
		
		_value_spin.value_changed.connect (() => notify_property ("value"));
		
		pack_value (_value_spin);
	}
	
	public void set_range (int min, int max, int step)
	{
		_value_spin.set_range ((double)min, (double)max);
		_value_spin.set_increments ((double)step, (double)step*5.0);
	}
}
