/* builderloader.vala
 *
 * Copyright (C) 2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

namespace I4uc.Core.BuilderLoader
{
	private enum TagType
	{
		BUILDER,
		SOURCE_TYPE,
		OUTPUT_TYPE
	}
	
	private Builder _builder;
	private TagType _tag_type;
	private string _type_name;
	private OutputCommand _output_command;
	private Option _new_option;
	private string _lang;
	
	public void load (Builder builder, string uri) throws GLib.Error
	{
		_builder = builder;
		var file = File.new_for_uri (uri);
		string contents;
		file.load_contents (null, out contents);
		MarkupParser parser = { parser_start, parser_end, parser_text, null, null };
		MarkupParseContext context = new MarkupParseContext (parser, 0, null, null);
		context.parse (contents, -1);
	}
	
	private void parser_start (MarkupParseContext context, string name, string[] attr_names, string[] attr_values) throws MarkupError
	{
		switch (name)
		{
			case "builder":
				_tag_type = TagType.BUILDER;
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "name":
							_builder.name = attr_values[i];
							break;
					}
				}
				break;
			
			case "source_type":
				_tag_type = TagType.SOURCE_TYPE;
				var source_type = new SourceType ();
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "name":
							source_type.name = attr_values[i];
							break;
						case "input_type":
							source_type.input_type = attr_values[i];
							break;
						case "output_type":
							source_type.output_type = attr_values[i];
							break;
						case "clean_types":
							foreach (var clean_type in attr_values[i].split (","))
								source_type.clean_types.add (clean_type);
							break;
					}
				}
				_builder.source_types[source_type.name] = source_type;
				_type_name = source_type.name;
				break;
			
			case "output_type":
				_tag_type = TagType.OUTPUT_TYPE;
				var output_type = new OutputType ();
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "name":
							output_type.name = attr_values[i];
							break;
						case "output_type":
							output_type.output_type = attr_values[i];
							break;
						case "clean_types":
							foreach (var clean_type in attr_values[i].split (","))
								output_type.clean_types.add (clean_type);
							break;
					}
				}
				_builder.output_types[output_type.name] = output_type;
				_type_name = output_type.name;
				break;
			
			case "command":
				if (_tag_type == TagType.SOURCE_TYPE)
				{
					for (int i=0; i<attr_names.length; i++)
					{
						switch (attr_names[i])
						{
							case "format":
								_builder.source_types[_type_name].command_format = attr_values[i];
								break;
						}
					}
				}
				else
				{
					_output_command = new OutputCommand ();
					for (int i=0; i<attr_names.length; i++)
					{
						switch (attr_names[i])
						{
							case "format":
								_output_command.format = attr_values[i];
								break;
						}
					}
					_builder.output_types[_type_name].commands.add (_output_command);
				}
				break;
			
			case "option":
				string option_name="", format="", value_type="";
				bool optional=true, most_used=true;
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "name":
							option_name = attr_values[i];
							break;
						case "format":
							format = attr_values[i];
							break;
						case "value_type":
							value_type = attr_values[i];
							break;
						case "optional":
							optional = attr_values[i].to_bool ();
							break;
						case "most_used":
							most_used = attr_values[i].to_bool ();
							break;
					}
				}
				if (value_type == "List")
					_new_option = new ListOption ();
				else if (value_type == "Range")
					_new_option = new RangeOption ();
				else
					_new_option = new Option ();
				_new_option.name = option_name;
				_new_option.format = format;
				_new_option.value_type = value_type;
				_new_option.optional = optional;
				_new_option.most_used = most_used;
				break;
			
			case "label":
			case "description":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "lang":
							_lang = attr_values[i];
							break;
					}
				}
				break;
				
			case "items":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "default":
							(_new_option as ListOption).default = attr_values[i];
							break;
					}
				}
				break;
			
			case "range":
				for (int i=0; i<attr_names.length; i++)
				{
					switch (attr_names[i])
					{
						case "min":
							(_new_option as RangeOption).min = attr_values[i].to_int ();
							break;
						case "max":
							(_new_option as RangeOption).max = attr_values[i].to_int ();
							break;
						case "step":
							(_new_option as RangeOption).step = attr_values[i].to_int ();
							break;
						case "default":
							(_new_option as RangeOption).default = attr_values[i].to_int ();
							break;
					}
				}
				break;
		}
	}
	
	private void parser_end (MarkupParseContext context, string name) throws MarkupError
	{
		switch (name)
		{
			case "option":
				if (_tag_type == TagType.SOURCE_TYPE)
					_builder.source_types[_type_name].options.add (_new_option);
				else
					_builder.output_types[_type_name].options.add (_new_option);
				break;
		}
	}
	
	private void parser_text (MarkupParseContext context, string text, size_t text_len) throws MarkupError
	{
		switch (context.get_element ())
		{
			case "devices":
				foreach (var device in text.split (","))
					_builder.devices.add (device);
				break;
			
			case "required_options":
				foreach (var required_option in text.split (","))
					if (_tag_type == TagType.SOURCE_TYPE)
						_builder.source_types[_type_name].command_required_options.add (required_option);
					else
						_output_command.required_options.add (required_option);
				break;
			
			case "optional_options":
				foreach (var optional_option in text.split (","))
					if (_tag_type == TagType.SOURCE_TYPE)
						_builder.source_types[_type_name].command_optional_options.add (optional_option);
					else
						_output_command.optional_options.add (optional_option);
				break;
			
			case "regex":
				if (_tag_type == TagType.SOURCE_TYPE)
					_builder.source_types[_type_name].command_regex = text;
				else
					_output_command.regex = text;
				break;
			
			case "label":
				_new_option.label[_lang] = text;
				break;
			
			case "description":
				_new_option.description[_lang] = text;
				break;
			
			case "depends":
				foreach (var depend in text.split (","))
					_new_option.depends.add (depend);
				break;
			
			case "items":
				foreach (var item in text.split (","))
					(_new_option as ListOption).items.add (item);
				break;
		}
    }
}
