/* commandrunner.vala
 *
 * Copyright (C) 2009-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */

public abstract class I4uc.Core.CommandRunner : GLib.Object
{
	protected Pid? _child_pid = null;
	protected IOChannel _stdout_channel = null;
	protected IOChannel _stderr_channel = null;
	private uint _child_watch_id = 0;
	
	public bool is_running { get { return _child_pid != null; } }
	
	public signal void finished (bool exited_ok, string standard_output, string standard_error);
	
	public void run (string command, string? working_directory = null) throws SpawnError
	{
		return_if_fail (!this.is_running);
		
		int stdo, stde;
		string[] argv;
		Shell.parse_argv (command, out argv);
		try
		{
			Process.spawn_async_with_pipes (working_directory, argv, null, SpawnFlags.SEARCH_PATH | SpawnFlags.DO_NOT_REAP_CHILD, null, out _child_pid, null, out stdo, out stde);
		}
		catch (SpawnError e)
		{
			_child_pid = null;
			throw e;
		}
		
		_child_watch_id = ChildWatch.add (_child_pid, on_child_watch);
		
		_stdout_channel = create_io_channel (stdo);
		_stderr_channel = create_io_channel (stde);
	}
	
	public void stop ()
	{
		return_if_fail (this.is_running);
		
		close_process ();
		
		var ctx = GLib.MainContext.default ();
		while (_child_watch_id != 0 && ctx.pending ())
			ctx.iteration (false);
	}
	
	protected abstract void close_process ();
	
	protected abstract IOChannel create_io_channel (int fd);

	private void on_child_watch (Pid pid, int status)
	{
		// Read standard output and standard error
		string str;
		size_t length;
		_stdout_channel.read_to_end (out str, out length);
		var standard_output = length==0 ? "" : str;
		_stderr_channel.read_to_end (out str, out length);
		var standard_error = length==0 ? "" : str;
		
		// Close channels and process
		_stdout_channel = null;
		_stderr_channel = null;
		Source.remove (_child_watch_id);
		Process.close_pid (pid);
		_child_watch_id = 0;
		_child_pid = null;
		
		this.finished (status == 0, standard_output, standard_error);
	}
}

namespace I4uc.Core
{
	//Defined in commandrunnerfactory.c
	public extern CommandRunner create_command_runner ();
}
