/* document.vala
 *
 * Copyright (C) 2010  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */

public class I4uc.Core.Document : GLib.Object
{
	private File _file;
	private string _etag;
	
	public string uri { owned get { return _file.get_uri (); } }
	
	public string path { owned get { return _file.get_path (); } }
	
	public string display_name
	{
		owned get
		{
			var file_info = _file.query_info (FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME, FileQueryInfoFlags.NONE, null);
			return file_info.get_display_name ();
		}
	}
	
	public string parse_name { owned get { return _file.get_parse_name (); } }
	
	public string folder { owned get { return _file.get_parent ().get_uri (); } }
	
	public bool exists { get { return _file.query_exists (null); } }
	
	public bool can_read
	{
		get
		{
			var file_info = _file.query_info (FILE_ATTRIBUTE_ACCESS_CAN_READ, FileQueryInfoFlags.NONE, null);
			return file_info.get_attribute_boolean (FILE_ATTRIBUTE_ACCESS_CAN_READ);
		}
	}
	
	public bool can_write
	{
		get
		{
			var file_info = _file.query_info (FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FileQueryInfoFlags.NONE, null);
			return file_info.get_attribute_boolean (FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
		}
	}
	
	public string content_type
	{
		owned get
		{
			var file_info = _file.query_info (FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE, FileQueryInfoFlags.NONE, null);
			return file_info.get_content_type ();
		}
	}
	
	public string encoding { set; get; default = "UTF-8"; }
	
	public string load_contents (string uri) throws GLib.Error
	{
		string contents;
		string new_etag;
		var file = File.new_for_uri (uri);
		file.load_contents (null, out contents, null, out new_etag);
		if (_file == null || !_file.equal (file))
			_file = file;
		_etag = new_etag;
		string utf8_contents;
		to_utf8 (contents, out utf8_contents);
		return utf8_contents;
	}
	
	public void save_contents (string contents, string? uri = null, bool force_save = true) throws GLib.Error
	{
		File file;
		if (_file == null || (uri != null && _file.get_uri () != uri))
			file = File.new_for_uri (uri);
		else
			file = _file;
		var buffer = from_utf8 (contents + "\n");
		string new_etag;
		string? etag = force_save ? null: _etag;
		file.replace_contents (buffer, buffer.size (), etag, false, FileCreateFlags.NONE, out new_etag, null);
		if (_file == null || (uri != null && _file.get_uri () != uri))
			_file = file;
		_etag = new_etag;
	}
	
	public bool get_modified ()
	{
		if (_file == null)
			return false;
		
		string current_etag = null;
		try
		{
			var file_info = _file.query_info (FILE_ATTRIBUTE_ETAG_VALUE, FileQueryInfoFlags.NONE, null);
			current_etag = file_info.get_etag ();
		}
		catch (GLib.Error e)
		{
			warning (e.message);
			return false;
		}
		return (current_etag != null && current_etag != _etag);
	}
	
	private void to_utf8 (string encoded_string, out string utf8_string) throws GLib.ConvertError
	{
		foreach (var charset in Encodings.CHARSETS)
		{
			try
			{
				utf8_string = convert (encoded_string, (ssize_t)encoded_string.size (), "UTF-8", charset);
				this.encoding = charset;
				return;
			}
			catch (ConvertError e)
			{
				continue;
			}
		}
		throw new GLib.ConvertError.FAILED (_("Error trying to convert the document to UTF-8"));
	}
	
	private string from_utf8 (string utf8_string) throws GLib.ConvertError
	{
		return convert (utf8_string, (ssize_t)utf8_string.size (), _encoding, "UTF-8");
	}
}
