/* documentslogic.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

public class I4uc.Core.DocumentsLogic : GLib.Object
{
	private App _app;
	private DocumentsView _view;
	private ArrayList <DocumentPageLogic> _page_logics = new ArrayList <DocumentPageLogic> ();
	private DocumentPageLogic _page_logic_selected;
	private DocumentPreferencesPageLogic _preferences_page_logic;
	private HashMap<string, DocumentOpener> _document_openers = new HashMap<string, DocumentOpener> ();
	
	public DocumentsLogic (App app)
	{
		_app = app;
		_view = _app.documents_view;
		
		_preferences_page_logic = new DocumentPreferencesPageLogic (_app, _page_logics);
		
		//configure view
		_view.save_sensitive = false;
		_view.save_as_sensitive = false;
		_view.print_sensitive = false;
		_view.undo_sensitive = false;
		_view.redo_sensitive = false;
		_view.cut_sensitive = false;
		_view.copy_sensitive = false;
		_view.paste_sensitive = false;
		_view.search_sensitive = false;
		_view.syntax_highlight_menu_sensitive = false;
		_view.save_all_sensitive = false;
		_view.close_all_sensitive = false;
		_view.previous_sensitive = false;
		_view.next_sensitive = false;
		
		//connect view signals
		_view.pages_panel.page_changed.connect (on_page_changed);
		_view.new_clicked.connect (on_new_clicked);
		_view.open_clicked.connect (on_open_clicked);
		_view.open_recent_clicked.connect ((uri) => open_document (uri));
		_view.save_clicked.connect (() => _page_logic_selected.save ());
		_view.save_as_clicked.connect (on_save_as_clicked);
		_view.print_clicked.connect (() => _page_logic_selected.print ());
		_view.page_setup_clicked.connect (() => _view.show_page_setup_dialog ());
		_view.undo_clicked.connect (() => _page_logic_selected.undo ());
		_view.redo_clicked.connect (() => _page_logic_selected.redo ());
		_view.cut_clicked.connect (() => _page_logic_selected.cut ());
		_view.copy_clicked.connect (() => _page_logic_selected.copy ());
		_view.paste_clicked.connect (() => _page_logic_selected.paste ());
		_view.search_clicked.connect (() => _page_logic_selected.show_search_bar ());
		_view.syntax_highlight_menu.language_id_changed.connect (on_language_id_changed);
		_view.save_all_clicked.connect (on_save_all_clicked);
		_view.close_all_clicked.connect (() => close_all ());
		_view.previous_clicked.connect (() => _view.pages_panel.previous_page ());
		_view.next_clicked.connect (() => _view.pages_panel.next_page ());
	}
	
	public void open_document (string document_uri, int line=-1)
	{
		foreach (var file_extension in _document_openers.keys)
			if (document_uri.has_suffix (file_extension) &&
			    _document_openers[file_extension].open_document (document_uri))
			    return;
				
		var page_logic = get_page_logic_from_title (document_uri);
		if (page_logic == null)
		{
			var page_view = _view.create_page_view ();
			page_logic = create_page_logic (page_view);
			page_logic.open (document_uri);
			_view.pages_panel.add_page (page_view);
		}
		_view.pages_panel.show_page (page_logic.view);
		if (line != -1)
			page_logic.view.go_to_line (line);
		page_logic.view.grab_focus ();
	}
	
	public void open_documents (Gee.List<string> document_uris)
	{
		foreach (var document_uri in document_uris)
			open_document (document_uri);
	}
	
	public void save_documents (Gee.List<string> document_uris)
	{
		foreach (var document_uri in document_uris)
		{
			var page_logic = get_page_logic_from_title (document_uri);
			if (page_logic != null)
				page_logic.save ();
		}
	}
	
	public bool close_all ()
	{
		var unsaved_documents = new HashMap <string, string> ();
		Gee.List<string> documents_to_save;
		foreach (var page_logic in _page_logics)
			if (page_logic.view.content_modified)
				unsaved_documents[page_logic.view.title] = page_logic.view.tab_title;
		
		if (unsaved_documents.size != 0)
		{
			var response = _view.show_close_confirmation_dialog (unsaved_documents, out documents_to_save);
			
			if (response == DialogResponse.CANCEL)
				return false;
			
			if (response == DialogResponse.OK)
				foreach (var document in documents_to_save)
				{
					var page_logic = get_page_logic_from_title (document);
					if (page_logic != null)
						page_logic.save ();
				}
		}
		
		foreach (var page_logic in _page_logics)
			_view.pages_panel.remove_page (page_logic.view);
		_page_logics.clear ();
		unselected_page_view ();
		return true;
	}
	
	public void add_document_opener (string file_extension, DocumentOpener document_opener)
	{
		_document_openers[file_extension] = document_opener;
	}
	
	private void on_page_changed (Page? page)
	{
		if (page == null || !(page is DocumentPageView))
			unselected_page_view ();
		else
			(page as DocumentPageView).selected ();
	}
	
	private void unselected_page_view ()
	{
		_page_logic_selected = null;
		_view.save_sensitive = false;
		_view.save_as_sensitive = false;
		_view.print_sensitive = false;
		_view.undo_sensitive = false;
		_view.redo_sensitive = false;
		_view.cut_sensitive = false;
		_view.copy_sensitive = false;
		_view.paste_sensitive = false;
		_view.search_sensitive = false;
		_view.syntax_highlight_menu_sensitive = false;
		if (_page_logics.size == 0)
		{
			_view.save_all_sensitive = false;
			_view.close_all_sensitive = false;
			_view.previous_sensitive = false;
			_view.next_sensitive = false;
		}
	}
	
	private void on_page_logic_selected (DocumentPageLogic page_logic)
	{
		_view.save_sensitive = page_logic.can_save;
		_view.save_as_sensitive = true;
		_view.print_sensitive = true;
		on_page_logic_refresh_actions (page_logic);
		_view.search_sensitive = true;
		_view.save_all_sensitive = true;
		_view.close_all_sensitive = true;
		var page_num = _view.pages_panel.get_page_num (page_logic.view);
		_view.previous_sensitive = page_num > 0;
		_view.next_sensitive = page_num < _view.pages_panel.get_num_of_pages () - 1 ;
		_page_logic_selected = page_logic;
		_view.syntax_highlight_menu_sensitive = true;
		_view.syntax_highlight_menu.language_id = page_logic.view.language_id;
	}
	
	private void on_page_logic_refresh_actions (DocumentPageLogic page_logic)
	{
		_view.undo_sensitive = page_logic.view.can_undo;
		_view.redo_sensitive = page_logic.view.can_redo;
		_view.cut_sensitive = page_logic.view.has_selection;
		_view.copy_sensitive = page_logic.view.has_selection;
		_view.paste_sensitive = true;
	}
	
	private void on_remove_page_logic (DocumentPageLogic page_logic)
	{
		_page_logics.remove (page_logic);
		_view.pages_panel.remove_page (page_logic.view);
	}
	
	private void on_new_clicked ()
	{
		var title = _("Unsaved file %i");
		int i = 0;
		while (get_page_logic_from_title (title.printf (++i)) != null)
			;
		var page_view = _view.create_page_view ();
		page_view.title = title.printf (i);
		page_view.tab_title = title.printf (i);
		page_view.tab_tooltip = "<b>%s</b> %s".printf (_("Name:"), title.printf (i));
		page_view.tab_image = TabImage.NEW;
		
		var page_logic = create_page_logic (page_view);
		page_logic.is_new = true;
		
		_view.pages_panel.add_page (page_view);
		_view.pages_panel.show_page (page_view);
		page_view.grab_focus ();
	}
	
	private void on_open_clicked ()
	{
		var folder_uri = _app.settings.working_folder;
		Gee.List<string> documents;
		_app.view_factory.show_open_dialog (ref folder_uri, out documents);
		_app.settings.working_folder = folder_uri;
		open_documents (documents);
	}
	
	private void on_save_as_clicked ()
	{
		_page_logic_selected.save_as ();
		_view.save_sensitive = _page_logic_selected.can_save;
	}
	
	private void on_language_id_changed ()
	{
		if (_page_logic_selected.view.language_id != _view.syntax_highlight_menu.language_id)
			_page_logic_selected.view.language_id = _view.syntax_highlight_menu.language_id;
	}
	
	private void on_save_all_clicked ()
	{
		foreach (var page_logic in _page_logics)
			if (page_logic.view.content_modified)
				page_logic.save ();
	}
	
	private DocumentPageLogic create_page_logic (DocumentPageView page_view)
	{
		var page_logic = new DocumentPageLogic (_app, page_view);
		page_logic.selected.connect (on_page_logic_selected);
		page_logic.refresh_actions.connect (on_page_logic_refresh_actions);
		page_logic.remove_document.connect (on_remove_page_logic);
		_page_logics.add (page_logic);
		return page_logic;
	}
	
	private DocumentPageLogic? get_page_logic_from_title (string title)
	{
		foreach (var page_logic in _page_logics)
			if (page_logic.view.title == title)
				return page_logic;
		return null;
	}
}
