/* profilebuilder.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

public class I4uc.Core.CommandContext : GLib.Object
{
	public string command;
	public string output;
	public string output_regex;
	public string source_uri;
}

public class I4uc.Core.ProfileBuilder : GLib.Object
{
	private App _app;
	private CommandRunner _command_runner = create_command_runner ();
	private ArrayList<CommandContext> _commands = new ArrayList<CommandContext> ();
	private int _command_index = 0;
	private Profile _profile;
	
	public bool is_building { private set; get; }
	public Project project { set; get; }

	public string profile
	{
		set { _profile = _project.profiles[value]; }
		get { return _profile.name; }
	}
	
	public signal void new_message (string message);
	public signal void new_error (string error);
	public signal void command_finished (CommandContext command_context);

	public ProfileBuilder (App app)
	{
		_app = app;
		_command_runner.finished.connect (on_command_runner_finished);
	}

	public void build () throws GLib.Error
	{
		if (_is_building)
			return;
		
		this.new_message (_("Start building \"%s\" profile of \"%s\" project...").printf (_profile.name, _project.name));
		
		var builder = _app.builders[_profile.builder];
		var objects = new ArrayList<string> ();
		
		_commands.clear ();
		foreach (var source in _profile.files)
		{
			var source_type = builder.get_source_type_from_filename (source);
			if (source_type == null)
				continue;
			
			var object = source[0:-source_type.input_type.length] + "-" + _profile.name + source_type.output_type;
			objects.add (object);
			
			var source_uri = project.get_file_uri (source);
			var object_uri = project.get_file_uri (object);
			
			// Build the source file if the object file doesn't exists, is older than the
			// source or if the profile's parameters where changed
			if (!FileUtils.test (Filename.from_uri (object_uri), FileTest.EXISTS) ||
			    check_mtime (source_uri, object_uri) ||
			    _profile.parameters_changed)
			{
				var command = source_type.command_format;
				command = command.replace ("%device", _profile.device);
				command = command.replace ("%source", Shell.quote (source));
				command = command.replace ("%object", object.replace (" ", "_"));
				command = command.replace ("%project", _project.name.replace (" ", "_"));
				command = command.replace ("%profile", _profile.name.replace (" ", "_"));
				
				// Add compile options if the source has it
				if (_profile.compile_options.has_key (source_type.name))
				{
					var option_settings = _profile.compile_options[source_type.name];
					var required_options = source_type.command_required_options;
					var optional_options = source_type.command_optional_options;
					var command_options = OptionsLogic.get_options (source_type.options, option_settings, required_options, optional_options, "%options");
					command = command.replace ("%options", command_options);
				}
				
				var command_context = new CommandContext ();
				command_context.command = command;
				command_context.output_regex = source_type.command_regex;
				command_context.source_uri = source_uri;
				
				_commands.add (command_context);
			}
		}
		
		// Restores parameters changed flag
		_profile.parameters_changed = false;
		
		if (objects.size == 0)
			return;
		
		var output_type = builder.output_types[_profile.output_type];
		var firmware = _project.name + "-" + _profile.name + output_type.output_type;
		var firmware_file = File.new_for_uri (_project.get_file_uri (firmware));

		if (firmware_file.query_exists (null) && _commands.size == 0)
		{
			this.new_message (_("There's nothing to build"));
			return;
		}

		var objects_string = "";
		foreach (var object in objects)
			objects_string +=  object.replace (" ", "_") + " ";
		
		foreach (var command_output in output_type.commands)
		{
			var command = command_output.format;
			command = command.replace ("%device", _profile.device);
			command = command.replace ("%objects", objects_string);
			command = command.replace ("%firmware", firmware.replace (" ", "_"));
			command = command.replace ("%project", _project.name.replace (" ", "_"));
			command = command.replace ("%profile", _profile.name.replace (" ", "_"));
			
			// Add link options
			if (_profile.link_options.has_key (output_type.name))
			{
				var option_settings = _profile.link_options[output_type.name];
				var required_options = command_output.required_options;
				var optional_options = command_output.optional_options;
				var command_options = OptionsLogic.get_options (output_type.options, option_settings, required_options, optional_options, "%options");
				command = command.replace ("%options", command_options);
			}
			
			var command_context = new CommandContext ();
			command_context.command = command;
			
			_commands.add (command_context);
		}
		
		on_command_runner_finished (true);
	}
	
	public void clean () throws GLib.Error
	{
		if (_is_building)
			return;

		this.new_message (_("Start cleaning \"%s\" profile of \"%s\" project...").printf (_profile.name, _project.name));
		
		var builder = _app.builders[_profile.builder];
		
		// Clean files generated during compilation
		foreach (var source in _profile.files)
		{
			var source_type = builder.get_source_type_from_filename (source);
			if (source_type == null)
				continue;
			
			foreach (var clean_type in source_type.clean_types)
			{
				var filename = source[0:-source_type.input_type.length] + "-" + _profile.name + clean_type;
				var file = File.new_for_uri (_project.get_file_uri (filename.replace (" ", "_")));
				if (file.query_exists (null))
				{
					this.new_message (_("Deleting %s").printf (file.get_path ()));
					file.delete (null);
				}
			}
		}
		
		// Clean files generated during link
		foreach (var clean_type in builder.output_types[_profile.output_type].clean_types)
		{
			var filename = _project.name + "-" + _profile.name + clean_type;
			var file = File.new_for_uri (_project.get_file_uri (filename.replace (" ", "_")));
			if (file.query_exists (null))
			{
				this.new_message (_("Deleting %s").printf (file.get_path ()));
				file.delete (null);
			}
		}
		
		this.new_message (_("The profile is clean"));
	}

	public void stop ()
	{
		_commands.clear ();
		_command_runner.stop ();
	}

	private void on_command_runner_finished (bool exited_ok, string standard_output="", string standard_error="")
	{
		// When _command_index is different to 0, a command just finished
		if (_command_index != 0)
		{
			_commands[_command_index-1].output = standard_output + standard_error;
			this.command_finished (_commands[_command_index-1]);
		}
		
		// Run the next command, exit when all the commands finished ok or with some error
		if (_command_index < _commands.size && exited_ok)
		{
			var folder_file = File.new_for_uri (_project.folder_uri);
			try
			{
				_command_runner.run (_commands[_command_index].command, folder_file.get_path ());
				this.is_building = true;
				_command_index++;
			}
			catch (Error e)
			{
				this.is_building = false;
				_command_index = 0;
				this.new_error (e.message);
			}
		}
		else
		{
			this.is_building = false;
			_command_index = 0;
			if (exited_ok)
				this.new_message (_("Build finished ok"));
			else
				this.new_error (_("Build finished with errors"));
		}
	}

	private bool check_mtime (string newer_uri, string older_uri) throws GLib.Error
	{
		var newer_file = File.new_for_uri (newer_uri);
		var older_file = File.new_for_uri (older_uri);
		
		var newer_info = newer_file.query_info ("time::*", FileQueryInfoFlags.NONE, null);
		var older_info = older_file.query_info ("time::*", FileQueryInfoFlags.NONE, null);
		
		var newer_time = newer_info.get_attribute_uint64 (FILE_ATTRIBUTE_TIME_CHANGED);
		var newer_time_usec = newer_info.get_attribute_uint32 (FILE_ATTRIBUTE_TIME_CHANGED_USEC);
		var older_time = older_info.get_attribute_uint64 (FILE_ATTRIBUTE_TIME_CHANGED);
		var older_time_usec = older_info.get_attribute_uint32 (FILE_ATTRIBUTE_TIME_CHANGED_USEC);
		
		return (newer_time > older_time) || (newer_time == older_time && newer_time_usec > older_time_usec);
	}
}
