/* programmerslogic.vala
 *
 * Copyright (C) 2010-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */

public class I4uc.Core.ProgrammersLogic : GLib.Object
{
	private App _app;
	private ProgrammersView _view;
	private DocumentsLogic _documents_logic;
	private Programmer _current_programmer;
	private CommandRunner _command_runner = create_command_runner ();
	private OptionsLogic _options_logic;
	private string _read_firmware;
	
	public bool can_write_flash { private set; get; }
	
	public ProgrammersLogic (App app)
	{
		_app = app;
		_view = _app.programmers_view;
		_documents_logic = _app.documents_logic;
		_options_logic = new OptionsLogic (_view.page.options_view);
		
		//Configure view
		_view.page.tab_title = _("Programmers");
		_view.log_page.tab_title = _("Programmer's log");
		
		//Load programmers
		foreach (var programmer in _app.programmers.values)
			_view.page.add_programmer (programmer.name);
		if (_app.settings.programmer in _app.programmers.keys)
			_view.page.current_programmer = _app.settings.programmer;
		on_programmer_changed ();
		
		//Connect signals
		_view.pages_panel.page_changed.connect (on_page_changed);
		_view.page.notify["current-programmer"].connect (on_programmer_changed);
		_view.page.update_programmer_clicked.connect (() => _options_logic.update_options ());
		_options_logic.option_value_changed.connect (on_options_logic_option_value_changed);
		_view.write_flash_clicked.connect (on_write_flash_clicked);
		_view.read_flash_clicked.connect (on_read_flash_clicked);
		_view.verify_flash_clicked.connect (() => run_command ("verify_flash", _current_programmer.verify_flash_command));
		_view.erase_flash_clicked.connect (on_erase_flash_clicked);
		_view.write_fuses_clicked.connect (on_write_fuses_clicked);
		_view.read_fuses_clicked.connect (() => run_command ("read_fuses", _current_programmer.read_fuses_command));
		_view.get_version_clicked.connect (() => run_command ("get_version", _current_programmer.get_version_command));
		_view.stop_clicked.connect (() => _command_runner.stop ());
		_command_runner.finished.connect (on_command_finished);
	}
	
	public void write_flash (string uri)
	{
		if (_app.view_factory.show_yes_no_message (_("Are you sure you want to write the flash memory?")) == DialogResponse.NO)
			return;
		
		try
		{
			var option_settings = _app.settings.programmers[_current_programmer.name].options;
			var required_options = _current_programmer.required_options["write_flash"];
			var optional_options = _current_programmer.optional_options["write_flash"];
			var filtered_command = OptionsLogic.get_options (_current_programmer.options, option_settings, required_options, optional_options, _current_programmer.write_flash_command);
			
			var firmware = File.new_for_uri (uri).get_path ();
			filtered_command = filtered_command.replace ("%firmware", Shell.quote (firmware));
		
			_view.log_page.add_message ("<b>" + _("Running:") + " </b><i>" + filtered_command + "</i>");
			_command_runner.run (filtered_command);
			set_parameters_sensitive (false);
			disable_actions ();
			_view.stop_sensitive = true;
		}
		catch (Error e)
		{
			_view.log_page.add_message (e.message);
		}
		
		_view.show_log_page ();
	}
	
	private void on_page_changed (Page? page)
	{
		disable_actions ();
		
		if (_view.page.current_programmer == null)
			return;
		
		enable_actions ();
	}
	
	private void on_programmer_changed ()
	{
		disable_actions ();
		
		if (_view.page.current_programmer == null)
		{
			_current_programmer = null;
			return;
		}
		
		_current_programmer = _app.programmers[_view.page.current_programmer];
		
		if (_app.settings.programmer != _current_programmer.name)
			_app.settings.programmer = _current_programmer.name;
		
		if (!_app.settings.programmers.has_key (_current_programmer.name))
		{
			_app.settings.programmers[_current_programmer.name] = new ProgrammerSettings ();
			_app.settings.programmers[_current_programmer.name].name = _current_programmer.name;
		}
		
		var option_settings = _app.settings.programmers[_current_programmer.name].options;
		_options_logic.add_options (_current_programmer.options, option_settings);
		
		enable_actions ();
	}
	
	private void on_options_logic_option_value_changed ()
	{
		disable_actions ();
		
		if (_view.page.current_programmer == null)
			return;
		
		enable_actions ();
	}
	
	private void on_write_flash_clicked ()
	{
		if (_app.view_factory.show_yes_no_message (_("Are you sure you want to write the flash memory?")) == DialogResponse.YES)
			run_command ("write_flash", _current_programmer.write_flash_command);
	}
	
	private void on_read_flash_clicked ()
	{
		string folder_uri = _app.settings.working_folder;
		string firmware_uri;
		var response = _app.view_factory.show_save_dialog (_("New firmware"), ref folder_uri, out firmware_uri);
		if (response == DialogResponse.OK)
		{
			_app.settings.working_folder = folder_uri;
			_read_firmware = firmware_uri;
			run_command ("read_flash", _current_programmer.read_flash_command);
		}
	}
	
	private void on_erase_flash_clicked ()
	{
		if (_app.view_factory.show_yes_no_message (_("Are you sure you want to erase the flash memory?")) == DialogResponse.YES)
			run_command ("erase_flash", _current_programmer.erase_flash_command);
	}
	
	private void on_write_fuses_clicked ()
	{
		if (_app.view_factory.show_yes_no_message (_("Are you sure you want to write the fuses?")) == DialogResponse.YES)
			run_command ("write_fuses", _current_programmer.write_fuses_command);
	}
	
	private void on_command_finished (bool exited_ok, string standard_output, string standard_error)
	{
		// Show programmer's output, adding a tab at the beginning of each line
		_view.log_page.add_message ("\t" + standard_output.replace ("\n", "\n\t") +
		                            "\t" + standard_error.replace ("\n", "\n\t"));
		
		// Show exit status message
		var exit_message = exited_ok ? _("Command finished ok") : _("Command finished with errors");
		_view.log_page.add_message ("<b>" + exit_message + "</b>");
		
		// Open the hex file if read flash was selected
		if (_read_firmware != null)
		{
			_documents_logic.open_document (_read_firmware);
			_read_firmware = null;
		}
		
		// Restore actions sensitive
		set_parameters_sensitive (true);
		enable_actions ();
	}
	
	private void run_command (string command, string format)
	{
		try
		{
			var option_settings = _app.settings.programmers[_current_programmer.name].options;
			var required_options = _current_programmer.required_options[command];
			var optional_options = _current_programmer.optional_options[command];
			var filtered_command = OptionsLogic.get_options (_current_programmer.options, option_settings, required_options, optional_options, format);
			
			if (_read_firmware == null)
			{
				if (_view.pages_panel.current_page != null &&
				    _current_programmer.is_valid_firmware_type (_view.pages_panel.current_page.tab_title))
				{
					var firmware = File.new_for_uri (_view.pages_panel.current_page.title).get_path ();
					filtered_command = filtered_command.replace ("%firmware", Shell.quote (firmware));
				}
			}
			else
			{
				var firmware = File.new_for_uri (_read_firmware).get_path ();
				filtered_command = filtered_command.replace ("%firmware", Shell.quote (firmware));
			}
			
			_view.log_page.add_message ("<b>" + _("Running:") + " </b><i>" + filtered_command + "</i>");
			_command_runner.run (filtered_command);
			set_parameters_sensitive (false);
			disable_actions ();
			_view.stop_sensitive = true;
		}
		catch (Error e)
		{
			_view.log_page.add_message (e.message);
		}
		
		_view.show_log_page ();
	}
	
	private void set_parameters_sensitive (bool sensitive)
	{
		_view.page.programmer_sensitive = sensitive;
		_view.page.options_view_sensitive = sensitive;
	}
	
	private void enable_actions ()
	{
		var valid_page = _view.pages_panel.current_page != null &&
		                 _current_programmer.is_valid_firmware_type (_view.pages_panel.current_page.tab_title);
		
		var write_flash_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["write_flash"]);
		var read_flash_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["read_flash"]);
		var verify_flash_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["verify_flash"]);
		var erase_flash_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["erase_flash"]);
		var write_fuses_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["write_fuses"]);
		var read_fuses_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["read_fuses"]);
		var get_version_valid_options = _options_logic.valid_required_options (_current_programmer.required_options["get_version"]);
		
		this.can_write_flash = _current_programmer.can_write_flash && write_flash_valid_options;
		_view.write_flash_sensitive = valid_page && _current_programmer.can_write_flash && write_flash_valid_options;
		_view.read_flash_sensitive = _current_programmer.can_read_flash && read_flash_valid_options;
		_view.verify_flash_sensitive = valid_page && _current_programmer.can_verify_flash && verify_flash_valid_options;
		_view.erase_flash_sensitive = _current_programmer.can_erase_flash && erase_flash_valid_options;
		_view.write_fuses_sensitive = _current_programmer.can_write_fuses && write_fuses_valid_options;
		_view.read_fuses_sensitive = _current_programmer.can_read_fuses && read_fuses_valid_options;
		_view.get_version_sensitive = _current_programmer.can_get_version && get_version_valid_options;
		_view.stop_sensitive = _command_runner.is_running;
	}
	
	private void disable_actions ()
	{
		this.can_write_flash = false;
		_view.write_flash_sensitive = false;
		_view.read_flash_sensitive = false;
		_view.verify_flash_sensitive = false;
		_view.erase_flash_sensitive = false;
		_view.write_fuses_sensitive = false;
		_view.read_fuses_sensitive = false;
		_view.get_version_sensitive = false;
		_view.stop_sensitive = false;
	}
}
