/* project.vala
 *
 * Copyright (C) 2009-2011  Matias De la Puente
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author:
 * 	Matias De la Puente <mfpuente.ar@gmail.com>
 */
using Gee;

public class I4uc.Core.Profile : GLib.Object
{
	private ArrayList<string> _files = new ArrayList<string> ();
	private HashMap<string, Gee.Map<string, OptionSettings>> _compile_options = new HashMap<string, Gee.Map<string, OptionSettings>> ();
	private HashMap<string, Gee.Map<string, OptionSettings>> _link_options = new HashMap<string, Gee.Map<string, OptionSettings>> ();
	
	public string name { set; get; }
	public bool is_public { set; get; default=true; }
	public string builder { set; get; default = ""; }
	public string output_type { set; get; default = ""; }
	public string device { set; get; default = ""; }
	public Gee.List<string> files { get { return _files; } }
	public Gee.Map<string, Gee.Map<string, OptionSettings>> compile_options { get { return _compile_options; } }
	public Gee.Map<string, Gee.Map<string, OptionSettings>> link_options { get { return _link_options; } }
	public bool parameters_changed { set; get; }
	
	public Profile.from_key_file (KeyFile key_file, string group) throws GLib.KeyFileError
	{
		_name = key_file.get_string (group, "name");
		if (key_file.has_key (group, "is_public"))
			_is_public = key_file.get_boolean (group, "is_public");
		_builder = key_file.get_string (group, "builder_id");
		_device = key_file.get_string (group, "device");
		
		var files = key_file.get_string_list (group, "files");
		_files.clear ();
		foreach (var file in files)
			_files.add (file);
	}
	
	public string to_xml (string prefix)
	{
		// Put profile's name and if it's public
		var xml_str = prefix + "<profile name=\"%s\" is_public=\"%s\">\n".printf (_name, _is_public.to_string ());
		
		// Put profile's files
		xml_str += prefix + "\t<files>\n";
		foreach (var file in _files)
			xml_str += prefix + @"\t\t<file>$file</file>\n";
		xml_str += prefix + "\t</files>\n";
		
		// Put profile's build options
		xml_str += prefix + "\t<build_options builder=\"%s\" output_type=\"%s\" device=\"%s\">\n".printf (_builder, _output_type, _device);
		foreach (var source_type in _compile_options.keys)
		{
			xml_str += prefix + "\t\t<compile_options source_type=\"%s\">\n".printf (source_type);
			foreach (var option_settings in _compile_options[source_type].values)
				xml_str += prefix + option_settings.to_xml ("\t\t\t");
			xml_str += prefix + "\t\t</compile_options>\n";
		}
		foreach (var output_type in _link_options.keys)
		{
			xml_str += prefix + "\t\t<link_options output_type=\"%s\">\n".printf (output_type);
			foreach (var option_settings in _link_options[output_type].values)
				xml_str += prefix + option_settings.to_xml ("\t\t\t");
			xml_str += prefix + "\t\t</link_options>\n";
		}
		xml_str += prefix + "\t</build_options>\n";
		
		xml_str += prefix + "</profile>\n";
		
		return xml_str;
	}
	
	public void set_link_options (Gee.Map<string, Gee.Map<string, OptionSettings>> link_options)
	{
		set_options (this.link_options, link_options);
	}
	
	public void set_compile_options (Gee.Map<string, Gee.Map<string, OptionSettings>> compile_options)
	{
		set_options (this.compile_options, compile_options);
	}
	
	private void set_options (Gee.Map<string, Gee.Map<string, OptionSettings>> out_options, Gee.Map<string, Gee.Map<string, OptionSettings>> in_options)
	{
		// Duplicate options settings to avoid bugs when duplicating or importing profiles
		foreach (var entry in in_options.entries)
		{
			var options = new HashMap<string, OptionSettings> ();
			foreach (var option in in_options[entry.key].values)
				options[option.name] = option.duplicate ();
			out_options[entry.key] = options;
		}
	}
}

public class I4uc.Core.Project : GLib.Object
{
	private const string _I4UC_PROJECT = "i4uc project";
	private uint _source_id = 0;
	
	private Document _project_file = new Document ();
	private ArrayList<string> _files = new ArrayList<string> ();
	private HashMap<string, Profile> _profiles = new HashMap<string, Profile> ();
	
	public string name { set; get; }
	public string uri { owned get { return _project_file.uri; } }
	public string path { owned get { return _project_file.path; } }
	public string folder_uri { owned get { return _project_file.folder; } }
	public Gee.List<string> files { get { return _files; } }
	public Gee.Map<string, Profile> profiles { get { return _profiles; } }
	
	public void open (string uri) throws GLib.Error
	{
		var project_content = _project_file.load_contents (uri);
		var key_file = new KeyFile ();
		try
		{
			key_file.load_from_data (project_content, project_content.length, KeyFileFlags.NONE);
		}
		catch (GLib.KeyFileError e)
		{
			// Try load the project in xml format
			ProjectLoader.load (this, project_content);
			return;
		}
		
		if (key_file.get_start_group () != _I4UC_PROJECT)
		{
			// Try load the project in xml format
			ProjectLoader.load (this, project_content);
			return;
		}
		
		this.name = key_file.get_string (_I4UC_PROJECT, "name");
		
		var files = key_file.get_string_list (_I4UC_PROJECT, "files");
		_files.clear ();
		foreach (var file in files)
			_files.add (file);
		
		_profiles.clear ();
		if (key_file.has_key (_I4UC_PROJECT, "profiles"))
			foreach (var profile in key_file.get_string_list (_I4UC_PROJECT, "profiles"))
				if (key_file.has_group (@"$profile profile"))
					_profiles[profile] = new Profile.from_key_file (key_file, @"$profile profile");
		
		// Save the project after opening in the old format. This saves the project in xml format
		save_contents ();
	}
	
	public void save (string? uri = null)
	{
		// The project is gonna be saved with the same name, don't try to save it
		if (uri == null && _source_id != 0)
			return;
		
		// Save the project if it's gonna have a new uri
		if (uri != null)
		{
			if (_source_id != 0)
			{
				Source.remove (_source_id);
				_source_id = 0;
			}
			save_contents (uri);
			return;
		}
		
		// Wait 2 seconds to save project, several build parameters change at almost the
		// same time
		_source_id = Timeout.add (2000, () => {
			save_contents (uri);
			_source_id = 0;
			return false;
		});
	}
	
	public string? get_file_uri (string file)
	{
		return File.new_for_uri (this.folder_uri).get_child_for_display_name (file).get_uri ();
	}
	
	private void save_contents (string? uri = null)
	{
		// The project IT'S gonna to be saved here
		
		// Put project's name
		var xml_str = "<project name=\"%s\">\n".printf (_name);
		
		// Put project's files
		xml_str += "\t<files>\n";
		foreach (var file in _files)
			xml_str += @"\t\t<file>$file</file>\n";
		xml_str += "\t</files>\n";
		
		// Put project's profiles
		xml_str += "\t<profiles>\n";
		foreach (var profile in _profiles.values)
			xml_str += profile.to_xml ("\t\t");
		xml_str += "\t</profiles>\n";
		
		xml_str += "</project>";
		
		_project_file.save_contents (xml_str, uri);
	}
}
